%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% StemGL_Fit Single stem Greenlab Fitting tool and
% StemGL_Sim Single stem GreenLab Simulation tool
%
%   MatLab_Octave code. 
%
% This code is part of GREENLAB project StemGL implementation
%
% Main Author: Philippe de Reffye CIRAD-AMAP
% Secondary author: Marc JAEGER CIRAD-AMAP
%
% Created: 2018
% Version 18_10_05
%
% Copyright (C) 2018 CIRAD-AMAP 
%
% This program is free software: you can redistribute it and/or modify it
% under the terms of the Create Commons Licence  type 5, BY-NC-SA
% BY (Attribution): Licensees may copy, distribute, display and perform the work and make derivative
% works and remixes based on it only if they give the author or licensor the credits (attribution)
% in the manner specified by these.
% SA (Share-alike): Licensees may distribute derivative works only under a license identical
% ("not more restrictive") to the license that governs the original work.
% NC (Non-commercial): 	Licensees may copy, distribute, display, and perform the work and
% make derivative works and remixes based on it only for non-commercial purposes.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [qo,Qmob_i,TQo_i] = Organ_GrowthFit(Q_D,qo,Qmob_i,i,b2,AxdPot,tfo,nmo,co,no,nf,po,kpo,Bo,Dlo,TQo_i,dim,offset)
% function  [qo,mxqo,Qmob_i,TQo_i] = Organ_GrowthFit(Q_D,qo,mxqo,Qmob_i,i,b2,AxdPot,tfo,nmo,co,no,nf,po,kpo,Bo,Dlo,TQo_i)
%
% Organ growth at cycle i
%   I:   Q_D:    Plant Production on demand ratio 
%           production at previous cycle Q(i -1) or seed Q*aQo / demand at cycle i
%   I:   i          : the cycle the computation applies to
%   I:   b2         : probability of the organ occurrence
%   I:   AcdPot(i)  : the potential axis of development
%   I:   tfo        : organ functioning duration time
%   I:   nmo        : remobilisation "intensity"
%   I:   co         : remobilisation coefficient (= 0: no remobilization)
%   I:   no(i)      : organ cohort occurences 
%   I:   nf(i)      : fruit cohort occurrences (make cause a feed back on sink multiplying it by kpo) 
%   I:   po         : sink function ratio (relative to leaf one)
%   I:   kpo        : sink ratio fruit feedback (if kpo > 0)
%   I:   Bo(j,Period): Sink variation BetaLaw 
%   I:   dim        : Table dimension 1  
%   I:   offset     : Table offset (for qo)
%   I&O: qo(i,j)    : organ production at cycle i when born at cycle j ( j <=i)
%   I&O: Qmob_i:    : Pool for remobilization at cycle i
%   I&O: TQo_i:     : Total organ biomass at cycle i
%
%

    if po > 0
        s = 1;
        for j = 1: i
            js = (s-1)*dim + j;
            ijs = (s-1)*dim*dim + (j-1) * dim + i; 
            dto = i-j+1;
            %% organ expansion
            if  dto-Dlo > 0
                if AxdPot(js) > 0 && no(js) > 0
                    dqi = po * Q_D * Bo(j,dto-Dlo);
                    if nf(js) > 0 && kpo > 0
                        dqi = dqi * kpo;
                    end
                else
                    dqi = 0;
                end
                if i == 1
                    % MJ 18/10/19 qo(i,j) = dqi;%  volume organe premier cycle
                    qo(ijs) = dqi;%  volume organe premier cycle
                else
                    % MJ 18/10/19 qo(i,j) = qo(i-1,j) + dqi ;         
                    qo(ijs) = qo(ijs-1) + dqi ;         
                end
                % MJ 18/10/16 mxqo(i,j) = qo(i,j);
            end
            % MJ 18/10/19 if qo(i,j) > 0
            if qo(ijs) > 0
                %% organ remobilisation
                if co > 0 && dto > tfo && i > 1
                    % MJ 18/10/16 qo(i,j) = mxqo(i,j) * (1-nmo*(1-(1-co)^(dto-tfo)));
                    % MJ 18/10/19 qo(i,j) = qo(i,j) * (1-nmo*(1-(1-co)^(dto-tfo)));
                    % MJ 18/10/19 Qmob_i  = Qmob_i + qo(i-1,j) - qo(i,j);
                    qo(ijs) = qo(ijs) * (1-nmo*(1-(1-co)^(dto-tfo)));
                    Qmob_i  = Qmob_i + qo(ijs-1) - qo(ijs);
                end
                % MJ 18/10/19 TQo_i = TQo_i + AxdPot(j)* b2 * no(j)* qo(i,j); % global compartment
                TQo_i = TQo_i + AxdPot(js)* b2 * no(js)* qo(ijs); % global compartment
                else
                    % MJ 18/10/19 qo(i,j) = 0.0000001; % organ always exists
                    qo(ijs) = 0.0000001; % organ always exists
            end
        end
    end
end